#pragma once

#include "beatlevel.h"

#include <nlohmann/json_fwd.hpp>
using json = nlohmann::json;

#include "filereader.h"

namespace Beatsaber {

class BeatLevelImpl : public BeatLevel, public std::enable_shared_from_this<BeatLevelImpl> {
protected:
	std::weak_ptr<BeatSet> parent;

	Difficulty::Difficulty dif;
	int32_t diffRank;
	std::string filename;

	double njs;
	double nso;

	const json& base;

	std::vector<Note> notes;
	std::vector<Wall> walls;

public:
	BeatLevelImpl(std::weak_ptr<BeatSet> p, std::shared_ptr<FileReader> r, const json& j);
	virtual ~BeatLevelImpl();

	virtual Difficulty::Difficulty getDifficulty() const override;
	virtual int32_t getDifficultyRank() const override;
	virtual std::string getFilename() const override;
	virtual double getNoteJumpSpeed() const override; // in m
	virtual double getNoteStartOffset() const override;
	virtual std::string getCustomData() const override;

	//TODO: get level content
	virtual const std::vector<Note>& getNotes() const override;
	virtual const std::vector<Wall>& getWalls() const override;

	virtual void printDebug() const override;

	virtual std::shared_ptr<BeatSet> getBeatSet() const override;
	virtual std::shared_ptr<BeatMap> getBeatMap() const override;
};

}